using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Feeds.Interop;

namespace RSSPlatformSample
{
    public partial class Form1 : Form
    {
        FeedsManager fm;
        IFeedFolder rootFolder;
        IFeedFolderEvents_Event fe;

        public Form1()
        {
            InitializeComponent();

            // Declaration des objets ncessaires
            fm = new FeedsManager();
            rootFolder = (IFeedFolder)fm.RootFolder;
            
            fe = (IFeedFolderEvents_Event)rootFolder.GetWatcher(FEEDS_EVENTS_SCOPE.FES_ALL, FEEDS_EVENTS_MASK.FEM_FEEDEVENTS | FEEDS_EVENTS_MASK.FEM_FOLDEREVENTS);
            // fe.FeedItemCountChanged += new IFeedFolderEvents_FeedItemCountChangedEventHandler(fe_FeedItemCountChanged);
            fe.FeedAdded += new IFeedFolderEvents_FeedAddedEventHandler(fe_FeedAdded);
            fe.FeedDeleted += new IFeedFolderEvents_FeedDeletedEventHandler(fe_FeedDeleted);
            fe.FeedRenamed += new IFeedFolderEvents_FeedRenamedEventHandler(fe_FeedRenamed);
            fe.FolderAdded += new IFeedFolderEvents_FolderAddedEventHandler(fe_FolderAdded);
                          
            DisplayFeeds();
        }

        #region Dfinition des FeedEvents handlers
        void fe_FolderAdded(string pbstrPath)
        {
            toolStripStatusLabel1.Text = "FolderAdded : on a besoin d'un petit Refresh..."; Application.DoEvents();
            System.Threading.Thread.Sleep(5*1000);
            DisplayFeeds();
        }

        void fe_FeedRenamed(string bstrPath, string bstrOldPath)
        {
            toolStripStatusLabel1.Text = "FeedRenamed : on a besoin d'un petit Refresh..."; Application.DoEvents();
            System.Threading.Thread.Sleep(5 * 1000);
            DisplayFeeds();
        }

        void fe_FeedDeleted(string bstrPath)
        {
            toolStripStatusLabel1.Text = "FeedDeleted : on a besoin d'un petit Refresh..."; Application.DoEvents();
            System.Threading.Thread.Sleep(5 * 1000);
            DisplayFeeds();
        }       

        void fe_FeedAdded(string bstrPath)
        {
            System.Threading.Thread.Sleep(5 * 1000);
            toolStripStatusLabel1.Text = "FeedAdded : on a besoin d'un petit Refresh..."; Application.DoEvents();
            DisplayFeeds();
        }
        #endregion

        #region fe_FeedItemCountChanged
        //void fe_FeedItemCountChanged(string bstrPath, int lItemCount, int lUnreadItemCount)
        //{
        //    // FeedItemCountChanged est dclanch ds que le nombre d'Items a chang,
        //    // soit directement parce que de nouveaux items sont prsents dans un flux donn
        //    // soit parce qu'un flux a t ajout ou supprim.
        //    // Dans notre cas, a ferait double emploi avec les autres vnements souscrits
        //    // throw new Exception("The method or operation is not implemented.");
        //    MessageBox.Show("FeedItemCountChanged: it looks like a Refresh is needed...");
        //    DisplayFeeds();
        //}
        #endregion

        private void DisplayFeeds()
        {
            if (listBox1.Items.Count > 0)
            {
                listBox1.Items.Clear();
                listView1.Items.Clear();
            }

            #region Parcours des folders et subfolders
            // loop through all the subfolders under the rootfolder
            foreach (IFeedFolder subFolder in (IFeedsEnum)rootFolder.Subfolders)
            {
                // DoNothing();
                // Dans cette dmo, par soucis de simplicit, j'ai choisi de ne traiter
                // que les flux du RootFolder et pas ceux des sous-dossiers ...
            }
            #endregion

            // loop through all the feeds in the rootfolder
            foreach (IFeed feed in (IFeedsEnum)rootFolder.Feeds)
            {
                listBox1.Items.Add(feed.name);
            }
            toolStripStatusLabel1.Text = "";
        }

        private void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            IFeed selectedFeed;
            IFeedsEnum items;

            try
            {
                selectedFeed = (IFeed) rootFolder.GetFeed(listBox1.SelectedItem.ToString());
                items = (IFeedsEnum)selectedFeed.Items;

                listView1.Items.Clear();

                foreach (IFeedItem itm in (IFeedsEnum)selectedFeed.Items)
                {
                    listView1.Items.Add(new ListViewItem(new string[] { itm.Title, itm.Link, itm.Description }));
                    webBrowser1.DocumentText = String.Empty;
                }
               
            }
            catch (Exception exc)
            {
                MessageBox.Show("Exception caught : "+ exc.Message, "Error");
            }
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            String fUrl = toolStripTextBox1.Text.Trim();

            if (toolStripTextBox1.Text.Trim().Length == 0)
            {
                // Wrong !
            }
            else
            {
                if (!fm.IsSubscribed(fUrl))
                {
                    IFeed newFeed;
                    // Le flux n'existe pas, on peut l'ajouter
                    rootFolder.CreateFeed("Nouveau flux  " + System.DateTime.Now.ToShortTimeString(), fUrl);
                    newFeed = (IFeed) fm.GetFeedByUrl(fUrl);
                }
            }
        }

        private void listView1_SelectedIndexChanged(object sender, EventArgs e)
        {            
            if (listView1.SelectedItems.Count > 0)
            {
                String webContent;

                ListViewItem itm = listView1.SelectedItems[0];
                String pTitle = (String)itm.SubItems[0].Text;
                String destUrl = (String)itm.SubItems[1].Text;
                String docText = (String)itm.SubItems[2].Text;

                webContent = "<font size=+1 face='Verdana,Arial,Tahoma'><a href=\""+destUrl+"\">"+pTitle+"</a></font><br/><br/>";
                webContent += docText;
                webBrowser1.DocumentText = webContent;
            }
            return;
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("A Simple Windows RSS Platform Sample Application, quickly and dirty written by Christophe Lauer - Microsoft France. http://blogs.microsoft.fr/clauer/", "Windows RSS Platform Sample Application");
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            fm = null;
            this.Close();
        }
    }
}